#include "TROOT.h"
#include "TFile.h"
#include "TCanvas.h"
#include "TRandom3.h"
#include "TH1D.h"
#include "TMath.h"

#include <iostream>
#include <sstream>

#include "CParticle.hh"
#include "CKinema.hh"

void Resolution()
{
  double u(931.494);

  int a1(1), a2(9);
  double p1_me(8.0713171);
  double p2_me(24.954264);
  CParticle p1(p1_me+u*a1),p2(p2_me+u*a2);

  double ein(200);
  double p1_sigma[] = {0.9288, 0.8853, 1.716};//130ps //200MeV

//  double ein(250);
//  double p1_sigma[] = {1.056, 1.016, 2.258};//130ps //250MeV

  //11Li
  //1g/cm2
//  double dele(0.04);
//  double estrag(0.9);
//  double multi_theta(1.3e-3);

  //2g/cm2
//  double dele(0.18);
//  double estrag(1.3);
//  double multi_theta(1.9e-3);

  //3g/cm2
//  double dele(0.42);
//  double estrag(1.6);
//  double multi_theta(2.4e-3);

  //4g/cm2
//  double dele(0.77);
//  double estrag(1.9);
//  double multi_theta(2.8e-3);

  //11Be
  //1g/cm2 //200MeV
//  double dele(29.355);
//  double estrag(1.3);
//  double multi_theta(1.8e-3);

  //2g/cm2 //200MeV
//  double dele(57.76);
//  double estrag(1.8);
//  double multi_theta(2.6e-3);

  //1g/cm2 //250MeV
//  double dele(24.69);
//  double estrag(1.0);
//  double multi_theta(1.1e-3);

  //2g/cm2 //250MeV
//  double dele(49.82);
//  double estrag(1.8);
//  double multi_theta(2.1e-3);

  //12B
  //1g/cm2 //200MeV
//  double dele(69.249);
//  double estrag(1.6);
//  double multi_theta(2.0e-3);

  //1g/cm2 //250MeV
//  double dele(56.61);
//  double estrag(1.6);
//  double multi_theta(1.7e-3);

  //1.5g/cm2 //250MeV
  double dele(85.43);
  double estrag(2.0);
  double multi_theta(2.1e-3);

  //2g/cm2 //250MeV
//  double dele(114.6);
//  double estrag(2.3);
//  double multi_theta(2.4e-3);

  p2.k(ein*a2);
  double p2_sigma[] = {p2.p()/700., p2.p()/700., p2.p()/700., dele/(ein*a2)/2*p2.p(), estrag, multi_theta};

  CParticle pv(u);
  pv.k(ein);
  double beta = pv.beta();

  //momentum spread, not dp/p -> this might change to dp/p not to be effected by K!!!
  //  double p1_sigma[] = {21./7.*2.88/sqrt(2.), 21./7.*2.88/sqrt(2.), 21./7.*3.03/sqrt(2.)};
  //  double p1_sigma[] = {10./7.*2.88/sqrt(2.), 10./7.*2.88/sqrt(2.), 10./7.*3.03/sqrt(2.)};
  //  double p1_sigma[] = {2.88/sqrt(2.), 2.88/sqrt(2.), 3.03/sqrt(2.)};
  //  double p1_sigma[] = {pv.p()/100./2.35, pv.p()/100./2.35, pv.p()/100./2.35};
  //  double p1_sigma[] = {1.12, 1.10, 2.1};//Hit4, Kai5, 130ps, 7m -> 172keV
  //  double p1_sigma[] = {1.28, 1.29, 2.9};//Hit4, Kai5, 200ps, 7m -> 202keV, 4.5%
  //  double p1_sigma[] = {1.57, 1.28, 3.32};//Hit3, Kai5, 200ps, 7m -> 221keV, 11%
  //  double p1_sigma[] = {1.84, 1.57, 4.4};//Hit3, Kai5, 200ps, 7m -> 266keV, 100%(3.47%(Erel=0.5MeV))
  //  double p1_sigma[] = {1.03, 1.03, 1.03};
  //  double p1_sigma[] = {0.0, 0.0, 0.0};
  //  double p2_sigma[] = {2.*5./3.*9.81, 2.*5./3.*9.81, 2.*3./3.*9.81, 0.0};
  //  double p2_sigma[] = {9.81, 9.81, 9.81, 14.653};
  //  double p2_sigma[] = {0.0, 0.0, 0.0, 0.0};

  TRandom3 random; 
  CKinema kinema;

  TFile* f = new TFile("root/temp.root", "recreate");
  TCanvas* c = gROOT->MakeDefCanvas();
  for(int j = 0; j < 8; ++j){
    double erel = 0.0125*pow(2,j);
    
    std::ostringstream oss;
    oss << "hist" << j;
    TH1D* h = new TH1D(oss.str().c_str(), "title", 300, 0.0, 2.0);
    for(int i = 0; i < 20000; ++i){
      p1.k(0);
      p2.k(0);

      double theta = acos(2*random.Uniform() - 1);
      double phi = TMath::Pi()*(2*random.Uniform() - 1);
      kinema.Fission(p1, p2, erel, theta, phi);
      kinema.LorentzBoost(p1, -beta, 2);
      kinema.LorentzBoost(p2, -beta, 2);

//      p1.p(p1.p(0)+random.Uniform(-p1_sigma[0]/2.,p1_sigma[0]/2.),
//	   p1.p(1)+random.Uniform(-p1_sigma[1]/2.,p1_sigma[1]/2.),
//	   random.Gaus(p1.p(2), p1_sigma[2]));
      p1.p(random.Gaus(p1.p(0), p1_sigma[0]),
	   random.Gaus(p1.p(1), p1_sigma[1]),
	   random.Gaus(p1.p(2), p1_sigma[2]));

      //energy stragglling
      p2.p(p2.p(0),
	   p2.p(1),
	   random.Gaus(p2.p(2), p2_sigma[4]));
      //multiple scattering and energy defference
      p2.p(random.Gaus(p2.p(0), p2_sigma[5]*p2.p(2)/sqrt(2)),
	   random.Gaus(p2.p(1), p2_sigma[5]*p2.p(2)/sqrt(2)),
	   p2.p(2)+random.Uniform(-p2_sigma[3]/2., p2_sigma[3]/2.));
      //detector resolution
      p2.p(random.Gaus(p2.p(0), p2_sigma[0]),
	   random.Gaus(p2.p(1), p2_sigma[1]),
	   random.Gaus(p2.p(2), p2_sigma[2]));

      double erel_exp = kinema.InvariantMass(p1, p2)-p1.m()-p2.m();

      h->Fill(erel_exp);
    }

    h->Draw();
    c->Update();

    h->Write();

    delete h;
  }
  f->Write();
  delete f;
  delete c;
}
