
#include "Angle.h"

void Angle(std::string filename)
{
  TCanvas* canvas = new TCanvas("canvas","",200,10,800,400);
  std::deque<TVector3> momentum;

  double u(931.494);
  int neutron_a(1);
  double neutron_me(8.0713171);
  CParticle neutron(u*neutron_a+neutron_me);

  int proton_a(1);
  double proton_me(7.28897050);
  CParticle proton(u*proton_a+proton_me);

  int carbon_a(12);
  double carbon_me(0);
  CParticle carbon(u*carbon_a+carbon_me);

  CKinema kinema;

  {
    TFile* file = TFile::Open(filename.c_str(), "READ");
    if(!file) return;

    TTree* tree;
    file->GetObject("Koyomi",tree);
    tree->Print();

    TFile* workfile = TFile::Open("root/temp.root","recreate");
    if(!workfile) return;

    TH1D* hplab = new TH1D("hplab", "theta", 100, 0., TMath::Pi());
    TH1D* hpcm = new TH1D("hpcm", "theta", 100, 0., TMath::Pi());

    CDataHimeSim* data = 0;
    TBranch *bdata = 0;
    if(tree->SetBranchAddress("hime",&data,&bdata)
       == TTree::kMissingBranch) return;

    Long64_t nentries = tree->GetEntriesFast();
    for(Long64_t i=0; i<nentries; i++) {
      if(i%100000 == 0) std::cout << "\r event:" << i << std::flush;

      Long64_t entry = tree->LoadTree(i);
      bdata->GetEntry(entry);

      std::deque<CDataHimeSim::OneData>::iterator itsim;
      itsim = data->fDequeOfOneData.begin();

      std::deque<CDataHimeSim::OneData>::iterator itmin
	= data->fDequeOfOneData.end();
      Double_t tmin = 1e+10;
      while(itsim != data->fDequeOfOneData.end()){
	if(itsim->fParticleName == "proton"){
	  if(tmin > itsim->fTime){
	    tmin = itsim->fTime;
	    itmin = itsim;
	  }
	}
	++itsim;
      }

      if(itmin != data->fDequeOfOneData.end()){
	neutron.p(data->fMomentum.x(),data->fMomentum.y(),data->fMomentum.z());
	proton.p(0, 0, 0);
	carbon.p(0, 0, 0);
	double beta = (neutron.p()+proton.p())/(neutron.e()+proton.e());
	//	double beta = (neutron.p()+carbon.p())/(neutron.e()+carbon.e());

	proton.p(itmin->fMomentum.x(),
		 itmin->fMomentum.y(),
		 itmin->fMomentum.z());
	kinema.LorentzBoost(proton, beta, 2);

	hplab->Fill(itmin->fMomentum.Theta());
	hpcm->Fill(proton.theta());
      }
    }
    std::cout << std::endl;

    canvas->Divide(2,1);
    canvas->cd(1);
    hplab->Draw();
    canvas->cd(2);
    hpcm->Draw();

    canvas->Update();
  }
}


