//
#ifndef CDataHime_h
#define CDataHime_h 1

#include <string>
#include <vector>
#include <deque>
#include <cmath>

#include "TString.h"
#include "TVector3.h"
#include "TLorentzVector.h"

#include "CData.hh"

class CDataHime : public CData
{
public:
  class OneData
  {
  public:
    OneData(std::vector<Int_t> id = std::vector<Int_t>(),
	    Double_t tl = 0., Double_t tr = 0.,
	    Double_t el = 0., Double_t er = 0., 
	    Double_t t = 0., Double_t e = 0.,
	    TVector3 x = TVector3())
      : fId(id), fTL(tl), fTR(tr), fAL(el), fAR(er), fT(t), fE(e), fX(x)
    {;}
    ~OneData(){;}

    void Print();

    std::vector<Int_t> fId;
    Double_t fTL;
    Double_t fTR;
    Double_t fAL;
    Double_t fAR;

    Double_t fT;
    Double_t fE;
    TVector3 fX;
  };

  std::deque<CDataHime::OneData> fDequeOfOneData;
 
  TLorentzVector fX;
  TLorentzVector fP;

  void CalMomentum()
  {
    double m0 = 939.56533;

    TVector3 beta = 1./(fX.T()*TMath::C()*1.e-7)*fX.Vect();
    TVector3 momentum = m0*1./sqrt(1.-beta.Mag()*beta.Mag())*beta;
    fP.SetVectM(momentum, m0);
  }

public:
  CDataHime();
  CDataHime(const char* name, const char* title = "");
  CDataHime(const TString& name, const TString& title = "");
  CDataHime(const CDataHime& himehit);
  CDataHime& operator=(const CDataHime& rhs);
  ~CDataHime();

  CDataHime::OneData* Find(Int_t idmother, Int_t id);
  void Reset();

  ClassDef(CDataHime, 1);
};

#endif


