// this file exists to re-define functions and make it easy to call.
//
#ifndef ALIAS_HH
#define ALIAS_HH

#include "TFitResult.h"
#include "TFitResultPtr.h"
#include <string>

//choose calculator
class CMyCalculator;
typedef CMyCalculator CCalculator;

class TFile;
class TTree;
class TBranch;
class TLeaf;
class TCanvas;
class TCut;
class TCutG;
class TH1;
class TProfile;
class TH1D;
class CHistGetter;
class CCutGetter;
class CCutGGetter;
class CCanvasGetter;

extern CHistGetter gHist;
extern CCutGetter gCut;
extern CCutGGetter gCutg;
extern CCanvasGetter gCanvas;

void fetch(std::string filename, int offset = -1);
void hstore(std::string filename);
void zone(Int_t nx = 1, Int_t ny = 1, Float_t xmargin = 0.01, Float_t ymargin = 0.01, Int_t color = 0);
void ls();
void ht(std::string name);
void ht(int i);
void ht(int id_begin, int id_end);
void hp();
void hb();
void hn();

void clear();

void xval();
void yval();
void xyval();
//void circle();
void zoom();

TFitResultPtr fit(const char* formula, Option_t* option = "", Option_t* goption = "", Double_t xmin = 0, Double_t xmax = 0);
TFitResultPtr fitp(int n = 1, Option_t* option = "", Option_t* goption = "", Double_t xmin = 0, Double_t xmax = 0);
TFitResultPtr fite(Option_t* option = "", Option_t* goption = "", Double_t xmin = 0,  Double_t xmax = 0);
TFitResultPtr fitg(Option_t* option = "", Option_t* goption = "", Double_t xmin = 0,  Double_t xmax = 0);
TFitResultPtr fitl(Option_t* option = "", Option_t* goption = "", Double_t xmin = 0,  Double_t xmax = 0);

TProfile* pfx(Int_t firstybin = 1, Int_t lastybin = -1, Option_t* option = "");
TProfile* pfy(Int_t firstxbin = 1, Int_t lastxbin = -1, Option_t* option = "");
TH1D* prx(Int_t firstybin = 0, Int_t lastybin = -1, Option_t* option = "");
TH1D* pry(Int_t firstxbin = 0, Int_t lastxbin = -1, Option_t* option = "");
TH1D* bnx(double firstx, double lastx, Option_t* option = "");
TH1D* bny(double firsty, double lasty, Option_t* option = "");

void calculate(std::string treename = "newtree", std::string treetitle = "title");

void setcanvas(TCanvas* canvas);
TCanvas* getcanvas();
void lgx();
void lgy();
void lgz();
void lnx();
void lny();
void lnz();
void divide(Int_t nx = 1, Int_t ny = 1, Float_t xmargin = 0.01, Float_t ymargin = 0.01, Int_t color = 0);
void cd(Int_t subpadnumber = 0);
void ncd();
void size(UInt_t ww, UInt_t wh);
void size();
void print(std::string filename = "", Option_t* option = "");

void readtree(std::string filename, std::string treename = "");
void writetree(std::string filename, std::string option = "NEW");
void readhist(std::string filename, int offset = -1);
void writehist(std::string filename, std::string option = "NEW");
void txttotree(std::string filename, std::string branchname, std::string treename = "temptree", std::string treetitle = "title");
void vectortotree(std::vector<std::vector<int> >& eventbuffer, std::string branchname, std::string treename, std::string treetitle);
TFile* getfile(std::string filename);
TFile* getfile(int id);
TFile* getfile();
void filels();

//void settree(TTree* tree);
TTree* gettree(std::string name);
TTree* gettree(int id);
TTree* gettree();
void treels();
void befriends();
void treetobranch();


TBranch* getbranch(std::string name);
TBranch* getbranch(int id);
TBranch* getbranch();
TLeaf* getleaf(std::string name);
TLeaf* getleaf(int id);
TLeaf* getleaf();
void pushbranch(std::string name, std::string title);
void branchls();
void branchht(std::string name, const TCut* selection = 0);
void branchht(int idx, int idy = -1, int idz = -1, const TCut* selection = 0);
void branchht(int idx, int idy, const TCut* selection);
void branchht(int idx, const TCut* selection);
void branchhp();
void branchhb();
void branchhn();

void clearhist();
void erasehist(int id);
void erasehist();
void erasehist(int ids, int idf);
TH1* gethist(std::string name);
TH1* gethist(int id);
TH1* gethist();
void inserthist(TFile* f, int offser = -1);
void histht(std::string name);
void histht(int i);
void histht(int id_begin, int id_end);
void histhp();
void histhb();
void histhn();
void histls();
void setdrawoption(std::string opt);
void setdrawoption();
void applyoption(std::string name);
void applyoption(int id);
void applyoption();
void applyoptionall();
void setfillcolor(Color_t fcolor);
void setfillcolor();

TCut* getcut(std::string name);
TCut* getcut(int id);
TCut* getcut();
TCutG* getcutg(std::string name);
TCutG* getcutg(int id);
TCutG* getcutg();
void cutls();

void createhist(int idx,
		Int_t nbinsx,
		Double_t xlow,
		Double_t xup,
		int idy,
		Int_t nbinsy,
		Double_t ylow,
		Double_t yup,
		int idz,
		Int_t nbinsz,
		Double_t zlow,
		Double_t zup,
		std::string name = "",
		const TCut* selection = 0);
void createhist(int idx,
		Int_t nbinsx,
		Double_t xlow,
		Double_t xup,
		int idy,
		Int_t nbinsy,
		Double_t ylow,
		Double_t yup,
		std::string histname = "",
		const TCut* selection = 0);
void createhist(int idx,
		Int_t nbinsx,
		Double_t xlow,
		Double_t xup,
		std::string histname = "",
		const TCut* selection = 0);
void createhist(std::string namex,
		std::string namey = "",
		std::string namez = "",
		std::string histname ="",
		const TCut* selection = 0);
void createhist(int idx,
		int idy = -1,
		int idz = -1,
		std::string histname = "",
		const TCut* selection = 0);
void createhistall();

void createcut(std::string name, std::string title);
void createcut(std::string name, TCut* cut);
void createcutg(std::string name = "cutg", std::string title = "cutg");
void circlecut(std::string name = "circlecut", std::string histname = "hist");

void convertrdftotxt(std::string InputFileName, std::string OutputFileName);
void convertrdftovector(std::string InputFileName, std::vector<std::vector<int> >& eventbuffer);

#endif

