#ifndef CDEQUEIDNAMELIST_HH
#define CDEQUEIDNAMELIST_HH

#include "TH1.h"

#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>
#include <deque>

template <class First, class Second>
class CDual
{
public:
  CDual(){;}
  CDual(First f, Second s) : first(f), second(s) {;}
  CDual(const CDual& right) : first(right.first), second(right.second){;}
  ~CDual(){;}

public:
  bool IsSameF(First f) const {return (f == first) ? true : false;}
  bool IsSameS(Second s) const {return (s == second) ? true : false;}

public:
  First first;
  Second second;
};

template<class First, class Second>
std::ostream& operator<<(std::ostream& os, const CDual<First, Second>& t);

template <class First, class Second, class Third> 
class CTriple  : public CDual<First, Second>
{
public:
  CTriple(){;}
  CTriple(First f, Second s, Third t) : CDual<First, Second>(f, s), third(t) {;}
  CTriple(const CTriple& right) : CDual<First, Second>(right.first, right.second), third(right.third) {;}
  ~CTriple(){;}

public:
  bool IsSameT(Third t) const {return (t == third) ? true : false;}

public:
  Third third;
};

template<class First, class Second, class Third>
std::ostream& operator<<(std::ostream& os, const CTriple<First, Second, Third>& t);

template <class First, class Second, class Third, class Fourth> 
class CQuad : public CTriple<First, Second, Third>
{
public:
  CQuad(){;}
  CQuad(First f, Second s, Third t, Fourth fo) : CTriple<First, Second, Third>(f, s, t), fourth(fo) {;}
  CQuad(const CQuad& right) : CTriple<First, Second, Third>(right.first, right.second, right.third), fourth(right.fourth) {;}
  ~CQuad(){;}

public:
  bool IsSameFo(Fourth fo) const {return (fo == fourth) ? true : false;}

public:
  Fourth fourth;
};

template<class First, class Second, class Third, class Fourth>
std::ostream& operator<<(std::ostream& os, const CQuad<First, Second, Third, Fourth>& t);

template <class First, class Second>
class CdequeDual : public std::deque< CDual<First, Second> >
{
public:
  CdequeDual() : std::deque<CDual<First, Second> >(){;}
  ~CdequeDual(){;}

  typename std::deque<CDual<First, Second> >::iterator FindF(First v);
  typename std::deque<CDual<First, Second> >::iterator FindS(Second v);
};

template <class First, class Second, class Third> 
class CdequeTriple : public std::deque< CTriple<First, Second, Third> >
{
public:
  CdequeTriple() : std::deque<CTriple<First, Second, Third> >(){;}
  ~CdequeTriple(){;}

  typename std::deque<CTriple<First, Second, Third> >::iterator FindF(First v);
  typename std::deque<CTriple<First, Second, Third> >::iterator FindS(Second v);
  typename std::deque<CTriple<First, Second, Third> >::iterator FindT(Third v);
};

template <class First, class Second, class Third, class Fourth> 
class CdequeQuad : public std::deque< CQuad<First, Second, Third, Fourth> >
{
public:
  CdequeQuad() : std::deque<CQuad<First, Second, Third, Fourth> >(){;}
  ~CdequeQuad(){;}

  typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator FindF(First v);
  typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator FindS(Second v);
  typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator FindT(Third v);
  typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator FindFo(Fourth v);
};

class CdequeIdNameTitle : public CdequeTriple<int, std::string, std::string>
{
public:
  CdequeIdNameTitle() : CdequeTriple<int, std::string, std::string>() {;}
  ~CdequeIdNameTitle(){;}
};

typedef CTriple<int, std::string, std::string> CIdNameTitle;


template<class First, class Second>
inline std::ostream& operator<<(std::ostream& os, const CDual<First, Second>& t)
{
  std::ostringstream oss;
  oss << "(" 
      << std::setw(3) << t.first << ", "
      << std::setw(3) << t.second << ")";
  return os << oss.str();
}

template <class First, class Second>
inline typename std::deque<CDual<First, Second> >::iterator
CdequeDual<First, Second>::FindF(First v)
{
  typename std::deque<CDual<First, Second> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameF(v)) break;
    ++it;
  }
  return it;
}

template <class First, class Second>
inline typename std::deque<CDual<First, Second> >::iterator
CdequeDual<First, Second>::FindS(Second v)
{
  typename std::deque<CDual<First, Second> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameS(v)) break;
    ++it;
  }
  return it;
}

template<class First, class Second, class Third>
inline std::ostream& operator<<(std::ostream& os, const CTriple<First, Second, Third>& t)
{
  std::ostringstream oss;
  oss << "(" 
      << std::setw(3) << t.first << ", "
      << std::setw(3) << t.second << ", "
      << std::setw(3) << t.third << ")";
  return os << oss.str();
}

//inline std::ostream& operator<<(std::ostream& os, const BranchId& t)
//{
//  std::ostringstream oss;
//  std::ostringstream ossf;
//  std::ostringstream osss;
//  std::ostringstream osst;
//
//  if(t.first < 0) ossf << "   ";
//  else ossf << t.first;
//  if(t.second < 0) osss << "   ";
//  else osss << t.second;
//  if(t.third < 0) osst <<  "   ";
//  else osst << t.third;
//
//  oss << "(" 
//      << std::setw(3) << ossf.str() << ", "
//      << std::setw(3) << osss.str() << ", "
//      << std::setw(3) << osst.str() << ")";
//  return os << oss.str();
//}

template <class First, class Second, class Third> 
inline typename std::deque<CTriple<First, Second, Third> >::iterator
CdequeTriple<First, Second, Third>::FindF(First v)
{
  typename std::deque<CTriple<First, Second, Third> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameF(v)) break;
    ++it;
  }
  return it;
}

template <class First, class Second, class Third> 
inline typename std::deque<CTriple<First, Second, Third> >::iterator
CdequeTriple<First, Second, Third>::FindS(Second v)
{
  typename std::deque<CTriple<First, Second, Third> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameS(v)) break;
    ++it;
  }
  return it;
}

template <class First, class Second, class Third> 
inline typename std::deque<CTriple<First, Second, Third> >::iterator
CdequeTriple<First, Second, Third>::FindT(Third v)
{
  typename std::deque<CTriple<First, Second, Third> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameT(v)) break;
    ++it;
  }
  return it;
}

template<class First, class Second, class Third, class Fourth>
inline std::ostream& operator<<(std::ostream& os, const CQuad<First, Second, Third, Fourth>& t)
{
  std::ostringstream oss;
  oss << "(" 
      << std::setw(3) << t.first << ", "
      << std::setw(3) << t.second << ", "
      << std::setw(3) << t.third << ", "
      << std::setw(3) << t.fourth << ")";
  return os << oss.str();
}

template <class First, class Second, class Third, class Fourth> 
inline typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator
CdequeQuad<First, Second, Third, Fourth>::FindF(First v)
{
  typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameF(v)) break;
    ++it;
  }
  return it;
}

template <class First, class Second, class Third, class Fourth> 
inline typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator
CdequeQuad<First, Second, Third, Fourth>::FindS(Second v)
{
  typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameS(v)) break;
    ++it;
  }
  return it;
}

template <class First, class Second, class Third, class Fourth> 
inline typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator
CdequeQuad<First, Second, Third, Fourth>::FindT(Third v)
{
  typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameT(v)) break;
    ++it;
  }
  return it;
}

template <class First, class Second, class Third, class Fourth> 
inline typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator
CdequeQuad<First, Second, Third, Fourth>::FindFo(Fourth v)
{
  typename std::deque<CQuad<First, Second, Third, Fourth> >::iterator it = this->begin();
  while(it != this->end()){
    if(it->IsSameFo(v)) break;
    ++it;
  }
  return it;
}

#endif
