#include "TROOT.h"

#include <iostream>
#include <sstream>

#include "CCanvasManager.hh"
#include "NGlobal.hh"

void CCanvasManager::SetCanvas(TCanvas* canvas)
{
  std::ostringstream os;
  os << FunctionName("SetCanvas") << "set canvas";
  NGlobal::Message(os.str(), kLOW);

  canvasname = canvas->GetName();

  initww = canvas->GetWw();
  initwh = canvas->GetWh();
}

void CCanvasManager::SetCanvas()
{
  std::ostringstream os;
  os << FunctionName("SetCanvas") << "unset canvas";
  NGlobal::Message(os.str(), kMID);

  canvasname = "";
}

TCanvas* CCanvasManager::GetCanvas()
{
  if(canvasname.empty()){
    if(gPad && gPad->InheritsFrom("TCanvas")){
      canvasname = gPad->GetName();
    }else{
      std::ostringstream os;
      os << FunctionName("GetCanvas") << "create defualt canvas";
      NGlobal::Message(os.str(), kLOW);

      //      TCanvas* canvas = new TCanvas;
      TCanvas* canvas = gROOT->MakeDefCanvas();
      canvas->SetFillColor(0);
      canvasname = canvas->GetName();
    }
    TObject* obj = gROOT->FindObject(canvasname.c_str());
    if(obj && obj->InheritsFrom("TCanvas")){
      SetCanvas((TCanvas*)obj);
    }
  }

  TObject* obj = gROOT->FindObject(canvasname.c_str());
  if(obj && obj->InheritsFrom("TCanvas")){
    return (TCanvas*)obj;
  }else{
    std::ostringstream os;
    os << FunctionName("GetCanvas") << "cannot get canvas, retry GetCanvas";
    NGlobal::Message(os.str(), kMID);

    canvasname = "";
    return GetCanvas();
  }
}

// set logscale function have to use gPad 
void CCanvasManager::SetLogx(int i)
{
  gPad->SetLogx(i);
}

void CCanvasManager::SetLogy(int i)
{
  gPad->SetLogy(i);
}

void CCanvasManager::SetLogz(int i)
{
  gPad->SetLogz(i);
}

void CCanvasManager::Divide(Int_t nx, Int_t ny, Float_t xmargin, Float_t ymargin, Int_t color)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  if(nx < 1 || ny < 1){
    std::ostringstream os;
    os << FunctionName("Divide") << "nx and ny must be positive number" ;
    NGlobal::Message(os.str(), kMID);

    return;
  }
  Clear();
  canvas->Divide(nx, ny, xmargin, ymargin, color);
  fsubpadnumber = nx*ny;
  cd(fsubpadnumber);
}

void CCanvasManager::cd(Int_t subpadnumber)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  subpadnumber = 1 + (subpadnumber - 1) % fsubpadnumber;
  canvas->cd(subpadnumber);
  fcurrentsubpadnumber = subpadnumber;
}

void CCanvasManager::NextSubpad()
{
  cd(++fcurrentsubpadnumber);
  fcurrentsubpadnumber %= fsubpadnumber;
}

void CCanvasManager::Resize(UInt_t ww, UInt_t wh)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  canvas->SetCanvasSize(ww, wh);
}

void CCanvasManager::Resize()
{
  Resize(initww, initwh);
}

void CCanvasManager::SaveAs(std::string filename, Option_t* option)
{
  TCanvas* canvas = GetCanvas();
  if(!canvas) return;

  canvas->SaveAs(filename.c_str(), option);
}

void CCanvasManager::Clear()
{
  if(!canvasname.empty()){
    TObject* obj = gROOT->FindObject(canvasname.c_str());
    if(obj && obj->InheritsFrom("TCanvas")){
      ((TCanvas*)obj)->Clear();
    }
    canvasname = "";
  }
  fcurrentsubpadnumber = 0;
  fsubpadnumber = 1;

  std::ostringstream os;
  os << FunctionName("Clear") << "cleared canvasmanager";
  NGlobal::Message(os.str(), kLOW);
}

std::string CCanvasManager::FunctionName(std::string functionname)
{
  return "[CCanvasManager::" + functionname + "]$ ";
}
