#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>

#include "GConverterRdfToTxt.hh"
#include "CConvertFile.hh"

#include "CConverterRdfToTxt.hh"

CConverterRdfToTxt::CConverterRdfToTxt() : fConvertFileIn(0), fConvertFileOut(0)
{
  /* nothing */
}

CConverterRdfToTxt::~CConverterRdfToTxt()
{
  /* nothing */
}

int CConverterRdfToTxt::SetFile(CConvertFile* file, std::string mode)
{
  if(mode == "in"){
    fConvertFileIn = file;
  }else if(mode == "out"){
    fConvertFileOut = file;
  }else{
    puts("CConverterRdfToTxt::set_file mode is wrong");
    return ERROR;
  }

  return OK;
}

int CConverterRdfToTxt::Convert()
{
  /* header block */
  {
    data_t header_block[BLOCK_SIZE/sizeof(data_t)];
    if(ReadHeader(header_block, fConvertFileIn) == ERROR){
      printf("read_heaader_binary\n");
      return ERROR;
    }
    fConvertFileOut->write_word("\n");
    WriteHeader(header_block, fConvertFileOut);
  }

  /* event block */
  {
    data_t event_block[BLOCK_SIZE/sizeof(data_t)];
    int i(0);
    /* read and write all event */
    while(1){
      /* check of ender block */
      fConvertFileIn->read_one_block(event_block);
      fConvertFileIn->seek_back_one_block();
      if((unsigned short)event_block[0] == FLAG_OF_ENDER){     
	break;
      }
      if(ReadEventBlock(event_block, fConvertFileIn) == ERROR){
	printf("ReadEventBlock\n");
	return ERROR;
      }
      ++i;
      WriteEventBlock(event_block, i, fConvertFileOut);
    }
  }
  /* ender block */
  {
    data_t ender_block[BLOCK_SIZE/sizeof(data_t)];
    if(ReadEnder(ender_block, fConvertFileIn) == ERROR){
      printf("ReadEnder\n");
      return ERROR;
    }
    WriteEnder(ender_block, fConvertFileOut);
  }

  return OK;
}

int CConverterRdfToTxt::ConvertOnMemory(std::vector<std::vector<int> >& eventbuffer)
{
  /* header block */
  {
    data_t header_block[BLOCK_SIZE/sizeof(data_t)];
    if(ReadHeader(header_block, fConvertFileIn) == ERROR){
      printf("read_heaader_binary\n");
      return ERROR;
    }
    PrintHeader(header_block);
  }

  /* event block */
  {
    data_t event_block[BLOCK_SIZE/sizeof(data_t)];
    int i(0);
    /* read and write all event */
    while(1){
      /* check of ender block */
      fConvertFileIn->read_one_block(event_block);
      fConvertFileIn->seek_back_one_block();
      if((unsigned short)event_block[0] == FLAG_OF_ENDER){     
	break;
      }
      if(ReadEventBlock(event_block, fConvertFileIn) == ERROR){
	printf("ReadEventBlock\n");
	return ERROR;
      }
      ++i;
      WriteEventBlock(event_block, i, eventbuffer);
      //      PrintEventBlock(event_block, i);
    }
    std::cout << std::endl;
  }

  /* ender block */
  {
    data_t ender_block[BLOCK_SIZE/sizeof(data_t)];
    if(ReadEnder(ender_block, fConvertFileIn) == ERROR){
      printf("ReadEnder\n");
      return ERROR;
    }
    PrintEnder(ender_block);
  }

  return OK;
}

/* rdf to readable rdf or txt */
int CConverterRdfToTxt::ReadHeader(data_t header_block[], CConvertFile* file)
{
  file->read_one_block(header_block);
  if((unsigned short)header_block[0] != FLAG_OF_HEADER){
    puts("this block is not header");
    return ERROR;
  }
  return OK;
}

int CConverterRdfToTxt::ReadEventBlock(data_t event_block[], CConvertFile* file)
{
  file->read_one_block(event_block);
  if((unsigned short)event_block[0] != FLAG_OF_EVENT_BLOCK){
    puts("this block is not event block");
    return ERROR;
  }
  return OK;
}

int CConverterRdfToTxt::ReadEnder(data_t ender_block[], CConvertFile* file)
{
  file->read_one_block(ender_block);
  if((unsigned short)ender_block[0] != FLAG_OF_ENDER){
    puts("this block is not ender");
    return ERROR;
  }
  return OK;
}

int CConverterRdfToTxt::WriteHeader(data_t header_block[], CConvertFile* file)
{
  char buf[OUTPUT_BUFFER_LENGTH];

  file->write_word("#  start header\n\n");

  /* run number */
  DecodeToString(header_block, buf, RUN_NUMBER_NUM, RUN_NUMBER_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");

  /* start time */
  DecodeToString(header_block, buf, START_TIME_NUM, START_TIME_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");

  /* stop time */
  DecodeToString(header_block, buf, STOP_TIME_NUM, STOP_TIME_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");

  /* print time */
  DecodeToString(header_block, buf, PRINT_TIME_NUM, PRINT_TIME_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");

  /* print date */
  DecodeToString(header_block, buf, PRINT_DATE_NUM, PRINT_DATE_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");

  /* header */
  DecodeToString(header_block, buf, HEADER_NUM, HEADER_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");

  /* ender */
  DecodeToString(header_block, buf, ENDER_NUM, ENDER_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");

  file->write_word("\n");
  return OK;
}

int CConverterRdfToTxt::PrintHeader(data_t header_block[])
{
  char buf[OUTPUT_BUFFER_LENGTH];

  std::cout << "\n#  start header\n" << std::endl;

  /* run number */
  DecodeToString(header_block, buf, RUN_NUMBER_NUM, RUN_NUMBER_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* start time */
  DecodeToString(header_block, buf, START_TIME_NUM, START_TIME_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* stop time */
  DecodeToString(header_block, buf, STOP_TIME_NUM, STOP_TIME_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* print time */
  DecodeToString(header_block, buf, PRINT_TIME_NUM, PRINT_TIME_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* print date */
  DecodeToString(header_block, buf, PRINT_DATE_NUM, PRINT_DATE_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* header */
  DecodeToString(header_block, buf, HEADER_NUM, HEADER_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* ender */
  DecodeToString(header_block, buf, ENDER_NUM, ENDER_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  std::cout << std::endl;

  return OK;
}

int CConverterRdfToTxt::WriteEventBlock(data_t event_block[], int i, CConvertFile* file)
{
  char buf[OUTPUT_BUFFER_LENGTH];
  char str[10];
  int data;
  size_t NOW_WORD_NUM(0);//現在のwordの位置

  file->write_word("#  ");
  file->write_word(i);
  file->write_word(" event block\n\n");

  NOW_WORD_NUM = FLAG_OF_EVENT_BLOCK_NUM;
  DecodeToString(event_block, buf,
		 NOW_WORD_NUM,
		 FLAG_OF_EVENT_BLOCK_LEN);/* 空読み */
  NOW_WORD_NUM += FLAG_OF_EVENT_BLOCK_LEN/sizeof(data_t);
  DecodeToString(event_block, buf,
		 NOW_WORD_NUM,
		 EVENT_BLOCK_BLANK_LEN);/* 空読み */
  NOW_WORD_NUM += EVENT_BLOCK_BLANK_LEN/sizeof(data_t);

  while(NOW_WORD_NUM < BLOCK_SIZE/sizeof(data_t) - 1){
    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, EVENT_SIZE_LEN);
    NOW_WORD_NUM += EVENT_SIZE_LEN/sizeof(data_t);
    int event_size = data;
    if((event_size & 0x8000) == 0x0000){
      break;//error check
    }
    event_size &= 0x7fff;//先頭の1ビットを落とす

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, FID_LEN);
    NOW_WORD_NUM += FID_LEN/sizeof(data_t);
    int fid = data;
    if(fid != 1){
      break;//error check
    }

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, EVENT_ID_LEN);
    NOW_WORD_NUM += EVENT_ID_LEN/sizeof(data_t);
    int event_id = data;
    sprintf(str, "%4d ", event_id);
    file->write_word(std::string(str));

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, SEGMENT_SIZE_LEN);
    NOW_WORD_NUM += SEGMENT_SIZE_LEN/sizeof(data_t);
    int segment_size = data;

    /* error check */
    if((event_size - segment_size) !=
       (EVENT_SIZE - SEGMENT_SIZE)/sizeof(data_t)){
      printf("event_size or segment_size is wrong\n");
      return ERROR;
    }

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, SEGMENT_ID_LEN);
    NOW_WORD_NUM += SEGMENT_ID_LEN/sizeof(data_t);
    int segment_id = data;
    sprintf(str, "%4d ", segment_id);
    file->write_word(std::string(str));

    for(size_t j = 0; j < segment_size - SEGMENT_SIZE/sizeof(data_t); ++j){ 
      DecodeToInteger(event_block, &data,
		      NOW_WORD_NUM, DATA_LEN);
      NOW_WORD_NUM += DATA_LEN/sizeof(data_t) ;
      sprintf(str, "%4d ", (unsigned short)data);
      file->write_word(std::string(str));
    }
    file->write_word("\n");
  }
  file->write_word("\n");
  return OK;
}

int CConverterRdfToTxt::WriteEventBlock(data_t event_block[], int i, std::vector<std::vector<int> >& eventbuffer)
{
  char buf[OUTPUT_BUFFER_LENGTH];
  int data;
  size_t NOW_WORD_NUM(0);//現在のwordの位置

  std::cout << "\r" << "#  " << i << " event block" << std::flush;

  NOW_WORD_NUM = FLAG_OF_EVENT_BLOCK_NUM;
  DecodeToString(event_block, buf,
		 NOW_WORD_NUM,
		 FLAG_OF_EVENT_BLOCK_LEN);/* 空読み */
  NOW_WORD_NUM += FLAG_OF_EVENT_BLOCK_LEN/sizeof(data_t);
  DecodeToString(event_block, buf,
		 NOW_WORD_NUM,
		 EVENT_BLOCK_BLANK_LEN);/* 空読み */
  NOW_WORD_NUM += EVENT_BLOCK_BLANK_LEN/sizeof(data_t);

  while(NOW_WORD_NUM < BLOCK_SIZE/sizeof(data_t) - 1){
    std::vector<int> event;

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, EVENT_SIZE_LEN);
    NOW_WORD_NUM += EVENT_SIZE_LEN/sizeof(data_t);
    int event_size = data;
    if((event_size & 0x8000) == 0x0000){
      break;//error check
    }
    event_size &= 0x7fff;//先頭の1ビットを落とす

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, FID_LEN);
    NOW_WORD_NUM += FID_LEN/sizeof(data_t);
    int fid = data;
    if(fid != 1){
      break;//error check
    }

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, EVENT_ID_LEN);
    NOW_WORD_NUM += EVENT_ID_LEN/sizeof(data_t);
    int event_id = data;
    event.push_back(event_id);

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, SEGMENT_SIZE_LEN);
    NOW_WORD_NUM += SEGMENT_SIZE_LEN/sizeof(data_t);
    int segment_size = data;

    /* error check */
    if((event_size - segment_size) !=
       (EVENT_SIZE - SEGMENT_SIZE)/sizeof(data_t)){
      printf("event_size or segment_size is wrong\n");
      return ERROR;
    }

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, SEGMENT_ID_LEN);
    NOW_WORD_NUM += SEGMENT_ID_LEN/sizeof(data_t);
    int segment_id = data;
    event.push_back(segment_id);

    for(size_t j = 0; j < segment_size - SEGMENT_SIZE/sizeof(data_t); ++j){ 
      DecodeToInteger(event_block, &data,
		      NOW_WORD_NUM, DATA_LEN);
      NOW_WORD_NUM += DATA_LEN/sizeof(data_t) ;
      event.push_back(data);
    }
    eventbuffer.push_back(event);
  }

  return OK;
}

int CConverterRdfToTxt::PrintEventBlock(data_t event_block[], int i)
{
  char buf[OUTPUT_BUFFER_LENGTH];
  int data;
  size_t NOW_WORD_NUM(0);//現在のwordの位置

  std::cout << "\n#  " << i << " event block\n" << std::endl;

  NOW_WORD_NUM = FLAG_OF_EVENT_BLOCK_NUM;
  DecodeToString(event_block, buf,
		 NOW_WORD_NUM,
		 FLAG_OF_EVENT_BLOCK_LEN);/* 空読み */
  NOW_WORD_NUM += FLAG_OF_EVENT_BLOCK_LEN/sizeof(data_t);
  DecodeToString(event_block, buf,
		 NOW_WORD_NUM,
		 EVENT_BLOCK_BLANK_LEN);/* 空読み */
  NOW_WORD_NUM += EVENT_BLOCK_BLANK_LEN/sizeof(data_t);

  while(NOW_WORD_NUM < BLOCK_SIZE/sizeof(data_t) - 1){
    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, EVENT_SIZE_LEN);
    NOW_WORD_NUM += EVENT_SIZE_LEN/sizeof(data_t);
    int event_size = data;
    if((event_size & 0x8000) == 0x0000){
      break;//error check
    }
    event_size &= 0x7fff;//先頭の1ビットを落とす

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, FID_LEN);
    NOW_WORD_NUM += FID_LEN/sizeof(data_t);
    int fid = data;
    if(fid != 1){
      break;//error check
    }

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, EVENT_ID_LEN);
    NOW_WORD_NUM += EVENT_ID_LEN/sizeof(data_t);
    int event_id = data;
    std::cout << event_id << " " << std::flush;

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, SEGMENT_SIZE_LEN);
    NOW_WORD_NUM += SEGMENT_SIZE_LEN/sizeof(data_t);
    int segment_size = data;

    /* error check */
    if((event_size - segment_size) !=
       (EVENT_SIZE - SEGMENT_SIZE)/sizeof(data_t)){
      printf("event_size or segment_size is wrong\n");
      return ERROR;
    }

    DecodeToInteger(event_block, &data,
		    NOW_WORD_NUM, SEGMENT_ID_LEN);
    NOW_WORD_NUM += SEGMENT_ID_LEN/sizeof(data_t);
    int segment_id = data;
    std::cout << segment_id << " " << std::flush;

    for(size_t j = 0; j < segment_size - SEGMENT_SIZE/sizeof(data_t); ++j){ 
      DecodeToInteger(event_block, &data,
		      NOW_WORD_NUM, DATA_LEN);
      NOW_WORD_NUM += DATA_LEN/sizeof(data_t) ;
      std::cout << data << " " << std::flush;
    }
    std::cout << std::endl;
  }

  return OK;
}

int CConverterRdfToTxt::WriteEnder(data_t ender_block[], CConvertFile* file)
{
  char buf[OUTPUT_BUFFER_LENGTH];

  file->write_word("#  start ender\n\n");

  /* run number */
  DecodeToString(ender_block, buf, RUN_NUMBER_NUM, RUN_NUMBER_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");
  /* start time */
  DecodeToString(ender_block, buf, START_TIME_NUM, START_TIME_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");
  /* stop time */
  DecodeToString(ender_block, buf, STOP_TIME_NUM, STOP_TIME_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");
  /* print time */
  DecodeToString(ender_block, buf, PRINT_TIME_NUM, PRINT_TIME_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");
  /* print date */
  DecodeToString(ender_block, buf, PRINT_DATE_NUM, PRINT_DATE_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");
  /* header */
  DecodeToString(ender_block, buf, HEADER_NUM, HEADER_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");
  /* ender */
  DecodeToString(ender_block, buf, ENDER_NUM, ENDER_LEN);
  file->write_word("#  ");
  file->write_word(std::string(buf));
  file->write_word("\n");

  file->write_word("\n");
  return OK;
}

int CConverterRdfToTxt::PrintEnder(data_t ender_block[])
{
  char buf[OUTPUT_BUFFER_LENGTH];

  std::cout << "\n#  start ender\n" << std::endl;  

  /* run number */
  DecodeToString(ender_block, buf, RUN_NUMBER_NUM, RUN_NUMBER_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* start time */
  DecodeToString(ender_block, buf, START_TIME_NUM, START_TIME_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* stop time */
  DecodeToString(ender_block, buf, STOP_TIME_NUM, STOP_TIME_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* print time */
  DecodeToString(ender_block, buf, PRINT_TIME_NUM, PRINT_TIME_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* print date */
  DecodeToString(ender_block, buf, PRINT_DATE_NUM, PRINT_DATE_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* header */
  DecodeToString(ender_block, buf, HEADER_NUM, HEADER_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  /* ender */
  DecodeToString(ender_block, buf, ENDER_NUM, ENDER_LEN);
  std::cout << "#  "  << std::string(buf) << std::endl;

  std::cout << std::endl;

  return OK;
}

int CConverterRdfToTxt::DecodeToString(data_t data_block[], char* str, int word_num, int word_len)
{
  memmove(&str[0], &data_block[word_num], word_len);
  memset(&str[word_len], '\0', 1);
  return 0;
}

int CConverterRdfToTxt::DecodeToInteger(data_t data_block[], int* data, int word_num, int word_len)
{
  memset(data, 0, sizeof(int));
  memmove(data, &data_block[word_num], word_len);/*アドレスの小さい方に桁の小さい数字が入っているからこれでOK*/
  return 0;
}
