#include "TROOT.h"
#include "TCutG.h"
#include "TCut.h"
#include "TSeqCollection.h"
#include "TCanvas.h"

#include <iostream>
#include <string>
#include <sstream>

#include "CCreateCut.hh"

#include "CCanvasManager.hh"
#include "CHistManager.hh"
#include "CCutManager.hh"
#include "CCreateHist.hh"
#include "CDynamicFunction.hh"
#include "NGlobal.hh"

void CCreateCut::CreateCut(std::string name, std::string title)
{
  std::ostringstream os;
  os << FunctionName("CreateCut") << "create cut";
  NGlobal::Message(os.str(), kMID);

  if(!NAlias::cutmanager->Check(name)) return;

  NGlobal::IncrementName(name);
  TCut* cut = new TCut(name.c_str(), title.c_str());
  NAlias::cutmanager->PushToList(cut);
  NAlias::cutmanager->Push(cut->GetName(), cut->GetTitle());
}

void CCreateCut::CreateCut(std::string name, TCut* cut)
{
  std::ostringstream os;
  os << FunctionName("CreateCut") << "create cut";
  NGlobal::Message(os.str(), kMID);

  if(!NAlias::cutmanager->Check(name)) return;

  NGlobal::IncrementName(name);
  TCut* newcut = new TCut(name.c_str(), cut->GetTitle());
  NAlias::cutmanager->PushToList(newcut);
  NAlias::cutmanager->Push(newcut->GetName(), newcut->GetTitle());
}

void CCreateCut::CreateCutG(std::string name, std::string title)
{
  std::ostringstream os;
  os << FunctionName("CreateCut") << "create cutg, click canvas, double click to quit";
  NGlobal::Message(os.str(), kMID);

  if(!NAlias::cutmanager->Check(name)) return;

  TCanvas* canvas = NAlias::canvasmanager->GetCanvas();
  if(!canvas) return;

  TCutG* cutg = (TCutG*)canvas->WaitPrimitive("CUTG", "CutG"); 
  if(!NAlias::cutmanager->Check(cutg)) return;

  NGlobal::IncrementName(name);
  cutg->SetName(name.c_str());
  cutg->SetTitle(title.c_str());
  NAlias::cutmanager->Push(cutg->GetName(), cutg->GetTitle());
}

void CCreateCut::CircleCut(std::string name, std::string histname)
{
  if(!NAlias::cutmanager->Check(name)) return;

  TH1* hist = NAlias::histmanager->GetHist();
  if(!NAlias::histmanager->Check(hist)) return;
  if(!hist->InheritsFrom("TH2")){
    std::ostringstream os;
    os << FunctionName("CircleCut") << name << " is not 2D histogram";
    NGlobal::Message(os.str(), kMID);

    return;
  }

  double x, y, r1, r2;
  NAlias::dynamicfunction->Circle(x, y, r1, r2);

  std::string namex = NAlias::histmanager->GetNameX();
  if(!NAlias::histmanager->Check(namex)) return;
  std::string namey = NAlias::histmanager->GetNameY();
  if(!NAlias::histmanager->Check(namey)) return;
  std::ostringstream os;
  os << "((" << namex << "-" << x << ")/" << r1 << ")^2" << "+"
     << "((" << namey << "-" << y << ")/" << r2 << ")^2" << "<1";

  CreateCut(name, os.str());
  NAlias::createhist->CreateHistFromBranch(namex, namey, "", histname, NAlias::cutmanager->GetCut());
  NAlias::canvasmanager->NextSubpad();
  NAlias::histmanager->DrawCurrent();
}

std::string CCreateCut::FunctionName(std::string functionname)  const
{
  return "[CreateCut::" + functionname + "]$ ";
}
