#include "TROOT.h"
#include "TCut.h"
#include "TH1.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TProfile.h"

#include <iostream>
#include <string>
#include <cstring>
#include <cstdio>

#include "CCanvasManager.hh"
#include "CHistManager.hh"
#include "CBranchManager.hh"
#include "NGlobal.hh"

#include "CCreateHist.hh"

void CCreateHist::CreateHistFromBranch(int idx,
				       Int_t nbinsx,
				       Double_t xlow,
				       Double_t xup,
				       int idy,
				       Int_t nbinsy,
				       Double_t ylow,
				       Double_t yup,
				       int idz,
				       Int_t nbinsz,
				       Double_t zlow,
				       Double_t zup,
				       std::string name,
				       const TCut* selection)
{
  std::string namex = NAlias::branchmanager->GetName(idx).c_str();
  if(!NAlias::branchmanager->Check(namex)) return;
  std::string namey = NAlias::branchmanager->GetName(idy).c_str();
  if(!NAlias::branchmanager->Check(namey)) return;
  std::string namez = NAlias::branchmanager->GetName(idz).c_str();
  if(!NAlias::branchmanager->Check(namez)) return;

  std::string histname;
  if(name.empty()){
    histname = namex + "_" + namey + "_" + namez;
  }else{
    histname = name.c_str();
  }

  if(selection){
    std::string selectionname = selection->GetName();
    histname += "_";
    histname += selectionname;
  }

  NGlobal::IncrementName(histname);

  std::string titlex = NAlias::branchmanager->GetName(idx).c_str();
  if(!NAlias::branchmanager->Check(titlex)) return;
  std::string titley = NAlias::branchmanager->GetName(idy).c_str();
  if(!NAlias::branchmanager->Check(titley)) return;
  std::string titlez = NAlias::branchmanager->GetName(idz).c_str();
  if(!NAlias::branchmanager->Check(titlez)) return;
  std::string histtitle = titlez + ":" + titley + ":" + titlex;

  new TH3D(histname.c_str(), histtitle.c_str(),
	   nbinsx, xlow, xup,
	   nbinsy, ylow, yup, 
	   nbinsz, zlow, zup);

  CreateHistFromBranchInnerFunc(histname, histtitle, selection);
}

void CCreateHist::CreateHistFromBranch(int idx,
				       Int_t nbinsx,
				       Double_t xlow,
				       Double_t xup,
				       int idy,
				       Int_t nbinsy,
				       Double_t ylow,
				       Double_t yup,
				       std::string name,
				       const TCut* selection)
{
  std::string namex = NAlias::branchmanager->GetName(idx).c_str();
  if(!NAlias::branchmanager->Check(namex)) return;
  std::string namey = NAlias::branchmanager->GetName(idy).c_str();
  if(!NAlias::branchmanager->Check(namey)) return;

  std::string histname;
  if(name.empty()){
    histname = namex + "_" + namey;
  }else{
    histname = name.c_str();
  }

  if(selection){
    std::string selectionname = selection->GetName();
    histname += "_";
    histname += selectionname;
  }

  NGlobal::IncrementName(histname);

  std::string titlex = NAlias::branchmanager->GetName(idx).c_str();
  if(!NAlias::branchmanager->Check(titlex)) return;
  std::string titley = NAlias::branchmanager->GetName(idy).c_str();
  if(!NAlias::branchmanager->Check(titley)) return;
  std::string histtitle = titley + ":" + titlex;

  new TH2D(histname.c_str(), histtitle.c_str(),
	   nbinsx, xlow, xup,
	   nbinsy, ylow, yup); 

  CreateHistFromBranchInnerFunc(histname, histtitle, selection);
}

void CCreateHist::CreateHistFromBranch(int idx,
				       Int_t nbinsx,
				       Double_t xlow,
				       Double_t xup,
				       std::string name,
				       const TCut* selection)
{
  std::string namex = NAlias::branchmanager->GetName(idx).c_str();
  if(!NAlias::branchmanager->Check(namex)) return;

  std::string histname;
  if(name.empty()){
    histname = namex.c_str();
  }else{
    histname = name.c_str();
  }

  if(selection){
    std::string selectionname = selection->GetName();
    histname += "_";
    histname += selectionname;
  }

  NGlobal::IncrementName(histname);

  std::string titlex = NAlias::branchmanager->GetName(idx).c_str();
  if(!NAlias::branchmanager->Check(titlex)) return;
  std::string histtitle = titlex.c_str();

  new TH1D(histname.c_str(), histtitle.c_str(),
	   nbinsx, xlow, xup);

  CreateHistFromBranchInnerFunc(histname, histtitle, selection);
}

void CCreateHist::CreateHistFromBranch(std::string namex,
				       std::string namey,
				       std::string namez,
				       std::string histname,
				       const TCut* selection)
{
  int idx(-1), idy(-1), idz(-1);
  std::string histtitle;
 
  if(!namex.empty()){
    idx = NAlias::branchmanager->GetId(namex);
    if(!NAlias::branchmanager->Check(idx)) return;
    std::string titlex = NAlias::branchmanager->GetName(idx);
    if(!NAlias::branchmanager->Check(titlex)) return;
    if(!namey.empty()){
      idy = NAlias::branchmanager->GetId(namey);
      if(!NAlias::branchmanager->Check(idy)) return;
      std::string titley = NAlias::branchmanager->GetName(idy);
      if(!NAlias::branchmanager->Check(titley)) return;
      if(!namez.empty()){
	idz = NAlias::branchmanager->GetId(namez);
	if(!NAlias::branchmanager->Check(idz)) return;
	std::string titlez = NAlias::branchmanager->GetName(idz);
	if(!NAlias::branchmanager->Check(titlez)) return;
	if(histname.empty()) histname = namex + "_" + namey + "_" + namez;
	if(selection) histname += "_" + std::string(selection->GetName());
	histtitle = titlez + ":" + titley + ":" + titlex;
      }else{
	if(histname.empty()) histname = namex + "_" + namey;
	if(selection) histname += "_" + std::string(selection->GetName());
	histtitle = titley + ":" + titlex;
      }
    }else{
      if(histname.empty()) histname = namex;
      if(selection) histname += "_" + std::string(selection->GetName());
	histtitle = titlex;
    }
  }else{
    std::ostringstream os;
    os << FunctionName("CreateHistFromBranch") << "no hist created";
    NGlobal::Message(os.str(), kMID);

    return;
  }

  NGlobal::IncrementName(histname);
  CreateHistFromBranchInnerFunc(histname, histtitle, selection);
}

void CCreateHist::CreateHistFromBranch(int idx,
				       int idy,
				       int idz,
				       std::string histname,
				       const TCut* selection)
{
  std::string histtitle;
  
  if(idx >= 0){
    std::string namex = NAlias::branchmanager->GetName(idx);
    if(!NAlias::branchmanager->Check(namex)) return;
    std::string titlex = NAlias::branchmanager->GetName(idx);
    if(!NAlias::branchmanager->Check(titlex)) return;
    if(idy >= 0){
      std::string namey = NAlias::branchmanager->GetName(idy);
      if(!NAlias::branchmanager->Check(namey)) return;
      std::string titley = NAlias::branchmanager->GetName(idy);
      if(!NAlias::branchmanager->Check(titley)) return;
      if(idz >= 0){
	std::string namez = NAlias::branchmanager->GetName(idz);
	if(!NAlias::branchmanager->Check(namez)) return;
	std::string titlez = NAlias::branchmanager->GetName(idz);
	if(!NAlias::branchmanager->Check(titlez)) return;
	if(histname.empty()) histname = namex + "_" + namey + "_" + namez;
	if(selection) histname += "_" + std::string(selection->GetName());
	histtitle = titlez + ":" + titley + ":" + titlex;
      }else{
	if(histname.empty()) histname = namex + "_" + namey;
	if(selection) histname += "_" + std::string(selection->GetName());
	histtitle = titley + ":" + titlex;
      }
    }else{
      if(histname.empty()) histname = namex;
      if(selection) histname += "_" + std::string(selection->GetName());
	histtitle = titlex;
    }
  }else{
    std::ostringstream os;
    os << FunctionName("CreateHistFromBranch") << "no hist created";
    NGlobal::Message(os.str(), kMID);

    return;
  }

  NGlobal::IncrementName(histname);
  CreateHistFromBranchInnerFunc(histname, histtitle, selection);
}

void CCreateHist::CreateHistFromBranchAll()
{
  CdequeIdNameTitle::iterator it = NAlias::branchmanager->Begin();
  bool init = true;
  CdequeIdNameTitle::iterator it_th1;
  while(it != NAlias::branchmanager->End()){
    CreateHistFromBranch(it->first);
    if(init) it_th1 = NAlias::histmanager->GetCurrent();
    init = false;
    ++it;
  }
}

void CCreateHist::CreateHistFromBranchInnerFunc(std::string histname, std::string histtitle, const TCut* selection)
{
  std::string varexp = histtitle + ">>" + histname;

  if(selection){
    std::ostringstream os;
    os << FunctionName("CreateHistFromBranch")
       << "create hist : " << varexp
       << ", cut : " << *selection;
    NGlobal::Message(os.str(), kMID);
  }else{
    std::ostringstream os;
    os << FunctionName("CreateHistFromBranch")
       << " create hist : " << varexp;
    NGlobal::Message(os.str(), kMID);
  }

  NAlias::canvasmanager->NextSubpad();
  NAlias::branchmanager->Draw(varexp, selection);
  NAlias::histmanager->Push(histname, histtitle);

  TH1* hist = NAlias::histmanager->GetHist();
  if(!NAlias::histmanager->Check(hist)) return;
  std::string title = hist->GetTitle();
  std::string namex = NAlias::histmanager->GetNameX();
  std::string namey = NAlias::histmanager->GetNameY();
  std::string namez = NAlias::histmanager->GetNameZ();
  std::string newtitle = title + ";" + namex + ";" + namey + ";" + namez;
  hist->SetTitle(newtitle.c_str());

  NAlias::histmanager->Draw(histname);
}

void CCreateHist::CreateHistFromCurrentHist(std::string histname, const TCut* selection)
{
  std::string title = NAlias::histmanager->GetTitle();
  if(!NAlias::branchmanager->Check(title)) return;
  std::string namex, namey, namez;
  NAlias::histmanager->GetNameXYZ("", namex, namey, namez);
  CreateHistFromBranch(namex, namey, namez, histname, selection);
}

std::string CCreateHist::FunctionName(std::string functionname)  const
{
  return "[CreateHist::" + functionname + "]$ ";
}
