
#include "TROOT.h"
#include "TFile.h"

#include "CDynamicFunction.hh"
#include "CSimpleFunction.hh"
#include "CTreeManager.hh"
#include "CBranchManager.hh"
#include "CHistManager.hh"
#include "CCutManager.hh"
#include "CCreateHist.hh"
#include "CCreateCut.hh"
#include "CFileManager.hh"
#include "CCanvasManager.hh"
#include "CMyCalculator.hh"

#include "CInitializer.hh"

//include only to know typedef of CCalculator
#include "Alias.hh"

namespace NAlias
{
  extern CDynamicFunction* dynamicfunction;
  extern CSimpleFunction* simplefunction;
  extern CCalculator* calculator;
  extern CFileManager* filemanager;
  extern CTreeManager* treemanager;
  extern CBranchManager* branchmanager;
  extern CHistManager* histmanager;
  extern CCutManager* cutmanager;
  extern CCreateHist* createhist;
  extern CCreateCut* createcut;
  extern CCanvasManager* canvasmanager;
}

CInitializer::CInitializer()
{
  // fTempFile = new TFile("temp.root", "UPDATE");
  fTempFile = new TFile("temp.root", "recreate");

  NAlias::simplefunction = new CSimpleFunction;
  NAlias::calculator = new CCalculator;
  NAlias::filemanager = new CFileManager;
  NAlias::treemanager = new CTreeManager;
  NAlias::branchmanager = new CBranchManager;
  NAlias::histmanager = new CHistManager;
  NAlias::cutmanager = new CCutManager;
  NAlias::createhist = new CCreateHist;
  NAlias::createcut = new CCreateCut;
  NAlias::canvasmanager = new CCanvasManager;
}

CInitializer::~CInitializer()
{
  delete NAlias::canvasmanager;
  delete NAlias::createcut;
  delete NAlias::createhist;
  delete NAlias::cutmanager;
  delete NAlias::histmanager;
  delete NAlias::branchmanager;
  delete NAlias::treemanager;
  delete NAlias::filemanager;
  delete NAlias::calculator;
  delete NAlias::simplefunction;
}

void CInitializer::cd()
{
  fTempFile->cd();
}

void CInitializer::Clear()
{
  fTempFile->Delete();

  std::ostringstream os;
  os << FunctionName("Clear") << "cleared initializer";
  NGlobal::Message(os.str(), kMID);
}

std::string CInitializer::FunctionName(std::string functionname) const
{
  return "[CInitializer::" + functionname + "]$ ";
}
