#ifndef CKINEMA_HH
#define CKINEMA_HH

#include "TROOT.h"
#include "TRandom3.h"

#include "CParticle.hh"

class CKinema
{
public:
  CKinema();
  ~CKinema();

public:
  //boost toward i=0(x),1(y),2(z)
  void LorentzBoost(CParticle& particle, double beta, int i);
  //2body fission by erel and angle
  void Fission(CParticle& particle1, CParticle& particle2, double erel, double theta, double phi);
  //2body fission by total energy
  void Fission(CParticle& particle1, CParticle& particle2, double etotal);
  //2body fission by erel and kin:incident energy par atomic mass unit
  void FissionErel(CParticle& particle1, CParticle& particle2, double erel, double kin);
  //derive kinematics of c and d by theta_lab and phi_lab(reaction:b(a,c)d)
  void Scatter(double theta_lab, double phi_lab, CParticle a, CParticle b, CParticle& c, CParticle& d);
  //used by Scatter, derive theta_cm by inputs
  double ThetaLabToThetaCm(double thetalab, double betacm, double betalorentz);
  //derive invariant mass of two particle
  double InvariantMass(const CParticle& particle1, const CParticle& particle2);

  class BetheBloch
  {
  public:
    static double Function(std::string pname, std::string tname, double kin);
    static double EnergyOut(std::string pname, std::string tname,
			    double ein, double rstop, double step);
    static double TimeOut(std::string pname, std::string tname,
			  CParticle& particle, double rstop, double step);

    static void ReadInParamater(std::string particle, std::string target,
			 double& M, double& z, double& Z, double& A,
			 double& I, double& C0, double& a, double& m,
			 double& X1, double& X0, double& rho);
  };

private:
  static TRandom3 random; 

  ClassDef(CKinema, 0)
};

#endif
