
#include <cmath>

#include "FFunction.hh"

Double_t VirtualResonance(Double_t *x, Double_t *par)
{
  Double_t xx = x[0];
  Double_t a  = par[0];
  Double_t r0 = par[1];
  Double_t Eb = par[2];
  Double_t ma = par[3];  
  Double_t mb = par[4];

  Double_t hbarc(197.0);

  Double_t mcn(ma*mb/(ma+mb));

  Double_t pcn(sqrt(xx*(xx+2.0*mcn)));
  Double_t k(sqrt(2.0*mcn*Eb));
  Double_t delta(atan(pcn/(-1.0/(a/hbarc)+0.5*(r0/hbarc)*pcn*pcn)));

  Double_t result = sqrt(pcn)*(cos(delta)+k*sin(delta)/pcn)/(k*k+pcn*pcn);

  return result*result;
}

Double_t BrietWigner(Double_t *x, Double_t *par)
{
  Double_t xx = x[0];
  Double_t er = par[0];
  Double_t gamma = par[1];

  Double_t g1(gamma*pow(er, -3.0/2.0));
  Double_t g2(0);//not exaxt;
  Double_t Gamma_Er(g1*pow(xx, 3.0/2.0)/(1+g2*xx));

  return Gamma_Er/((er-xx)*(er-xx)+0.25*Gamma_Er*Gamma_Er);
}
